'
' SPI Master demo to write and read RAM in the SPI slave
'
' Master                 Slave
' SDO(RC5) ------------> SDI(RC4)
' SDI(RC4) <------------ SDO(RC5)
' CLK(RC3) ------------> CLK(RC3)
' CS(RA5)  ------------> SS(RA5)
'
    Include "Amicus18.inc"          ' Use an 18F25K20 operating at 64MHz
'
' Variables
'
    Dim SPI_bAddress As Byte        ' Holds the address of the RAM to write/read
    Dim SPI_ByteOut As Byte         ' Holds the byte to write
    Dim SPI_ByteIn As Byte          ' Holds the byte read
'
' Setup the SPI peripheral's pins
'
    Symbol SPI_CLK = PORTC.3
    Symbol SPI_SDI = PORTC.4
    Symbol SPI_SDO = PORTC.5
    Symbol SPI_CS = PORTA.5
'
' Mnemonics
'
    Symbol cMnWrite = $F0           ' Write command
    Symbol cMnRead = $0F            ' Read command

'-----------------------------------------------------------------------------------
    GoTo Main                       ' Jump over any subroutines
'-----------------------------------------------------------------------------------
' Macro Name     : SPI_Write
' Return Value   : WREG holds the value read from the SPI interface
' Parameter      : Single 8-bit variable for SPI bus
' Description    : Write a single byte (8-bits) to the SPI bus
'
$define SPI_Write(pDataOut) '
    WREG = pDataOut         '
    _SPI_Write

_SPI_Write Macro-
    GoSub __SPI_Write
Endm

#ifMacro- _SPI_Write
__SPI_Write:
    SSPSTATBits_BF = 0              ' Empty the SSPBUF buffer flags
    PIR1bits_SSPIF = 0              ' Clear the interrupt flag
    SSPBUF = WREG                   ' Place the data to send into SSPBUF
    Clrwdt                          ' Keep the watchdog happy
    Btfss PIR1bits_SSPIF            ' \
    Bra $ - 2                       ' / Wait until cycle complete
    WREG = SSPBUF                   ' Read SSPBUF into WREG
    Return
#endIfMacro-
'-----------------------------------------------------------------------------------

$define SPI_Enable() Clear SPI_CS
$define SPI_Disable() Set SPI_CS

'-----------------------------------------------------------------------------------
' Write a byte to the slave
' Input     : pAddress holds the 8-bit address
'           : pValue holds the byte to write
' Output    : None
' Notes     : Has a small delay to give time for the SPI slave to write the value
'
$define Command_Write(pAddress, pValue) '
    SPI_Enable()                        '
    SPI_Write(cMnWrite)                 '
    SPI_Write(pAddress)                 '
    SPI_Write(pValue)                   '
    DelayCS 50                          '
    SPI_Disable()

'-----------------------------------------------------------------------------------
' Read a byte from the slave
' Input     : pAddress holds the 8-bit address
' Output    : pResult holds the byte read from the slave
' Notes     : Has a small delay to give time For the SPI slave to read the value
'
$define Command_Read(pAddress, pResult) '
    SPI_Enable()                        '
    SPI_Write(cMnRead)                  '
    SPI_Write(pAddress)                 '
    DelayCS 50                          '
    SPI_Write($FF)                      '
    pResult = WREG                      '
    SPI_Disable()
'-----------------------------------------------------------------------------------
' Deliberately send a malformed SPI command
' This will test the reset mechanism of the slave SPI
'
$define Command_Wrong(pAddress) '
    SPI_Enable()                '
    SPI_Write(pAddress)         '
    SPI_Disable()

'----------------------------------------------------------------------------
Main:
    DelayMS 100                                 ' Give time for the slave to power up

    SSPCON1 = $02                               ' Setup SPI for master mode, Fosc / 64 (1MHz operation)
    SSPCON1bits_CKP = 0                         ' Clock Polarity Select : Clock idle low
    SSPSTATbits_CKE = 0                         ' Clock Edge Select Bit : Transmit on idle to active transition
    SSPSTATbits_SMP = 0                         ' Data Input sample phase: Sample in middle of data
    PIR1bits_SSPIF = 0                          ' Clear buffer full status

    Output SPI_CLK                              ' Clock pin to output
    Input SPI_SDI                               ' Data in pin to input
    Output SPI_SDO                              ' Data out pin to output
    High SPI_CS                                 ' Disable the SPI slave
    SSPCON1bits_SSPEN = 1                       ' Enable SPI

    SPI_bAddress = 0
    While                                               ' Create an infinite loop
        HRSOut "Writing\r"
        For SPI_ByteOut = 255 To 0 Step -1              ' Create a loop to write to the slave's RAM
            Command_Write(SPI_bAddress, SPI_ByteOut)    ' Write to the slave's RAM
            Inc SPI_bAddress                            ' Move up the Address
        Next                                            ' Close the loop

        HRSOut "\rReading\r"
        For SPI_bAddress = 0 To 255                     ' Create a loop to read from the slave's RAM
            If SPI_bAddress // 10 = 1 Then              ' Have we reached the 10th read?
                Command_Wrong(SPI_bAddress)             ' Yes. So send a malformed command for testing
                HRSOut "Incorrect Command Sent\r"       ' Signal that it has been sent
            Else                                        ' Otherwise...
                Command_Read(SPI_bAddress, SPI_ByteIn)  ' Read from the slave's RAM
                HRSOut "Value = ", Dec SPI_ByteIn, 13   ' Display what was read
            EndIf
        Next                                            ' Close the loop
    Wend                                                ' Do it forever





