'
' Demonstrate an SPI slave to read and write a block of RAM
'
' Master                 Slave
' SDO(RC5) ------------> SDI(RC4)
' SDI(RC4) <------------ SDO(RC5)
' CLK(RC3) ------------> CLK(RC3)
' CS(RA5)  ------------> SS(RA5)
'
    Include "Amicus18.inc"                  ' Use an 18F25K20 operating at 64MHz

    On_Hardware_Interrupt GoTo SPI_Interrupt ' Point the interrupt to its handler
'
' Variables
'
    Dim SPI_SlaveFlag As Byte System
    Dim SPI_tDataReady As SPI_SlaveFlag.0   ' Signals that the SPI has data
    Dim SPI_bByteCount As Byte System       ' Counts the amount of bytes received
    Dim SPI_bValue As Byte System           ' Holds the value to write to RAM

    Dim SPI_bMnemonic As Byte               ' Holds the menmonic for read or write
    Dim SPI_bAddress As Byte                ' Holds the address of the RAM to write/read
    Dim bRAM[255] As Byte                   ' A block of RAM to read/write
'
' Setup the SPI peripheral's pins
'
    Symbol SPI_CLK = PORTC.3
    Symbol SPI_SDI = PORTC.4
    Symbol SPI_SDO = PORTC.5
    Symbol SPI_SS = PORTA.5
'
' Mnemonics
'
    Symbol cMnWrite = $F0                   ' Write command
    Symbol cMnRead = $0F                    ' Read command

$ifndef True
    $define True 1
$endif
$ifndef False
    $define False 0
$endif
'--------------------------------------------------------------------------

    GoTo Main                               ' Jump over the subroutines
'--------------------------------------------------------------------------
' Interrupt handler
' Reads mnemonic, address and either writes or reads from a RAM block
'
SPI_Interrupt:
    Context Save
    If PIR1bits_SSPIF = 1 Then              ' Is it an SPI interrupt?
        While SSPSTATbits_BF = 0: Wend      ' Yes. So wait for the byte
        SPI_bValue = SSPBUF
        Inc SPI_bByteCount                  ' Increment the byte counter
        SSPCON1bits_WCOL = 0                ' \
        SSPCON1bits_SSPOV = 0               ' / Clear any possible errors
        SPI_tDataReady = True               ' Signal that there's data to be worked upon
        PIR1bits_SSPIF = 0                  ' Reset the SPI interrupt flag
    EndIf
    Context Restore

'--------------------------------------------------------------------------
' Initialise the SPI slave routines
'
SPI_SlaveInit:
    INTCON = $00                ' Make sure interrupts are disabled
    WREG = SSPBUF               ' Clear the SSPBUF of any erroneous data
'
' Setup the SPI peripheral
'
    SSPCON1 = $04               ' Disable SPI and configure SCK, SDO, SDI and SS as SPI port pins
                                ' Idle state for clock is a low level
                                ' SPI Slave mode, clock = SCK pin, SS pin control enabled
    SSPSTAT = $00               ' Input data sampled at middle of data output time
                                ' Transmit occurs on transition from Idle to active clock state
'
' SPI slave pin configurations
'
    Input SPI_CLK               ' SCK as input
    Input SPI_SDI               ' SDI as input
    Output SPI_SDO              ' SDO as output
    Input SPI_SS                ' SS an input

    SPI_tDataReady = False      ' Clear the data ready flag
    Clear SPI_bByteCount
    Clear SPI_bValue
    Clear bRAM                  ' Clear the array
'
' Setup and enable the SPI interrupt mechanism
'
    PIR1bits_SSPIF = 0          ' Clear the SPI interrupt flag
    SSPCON1bits_SSPEN = 1       ' Enable the SPI peripheral
    PIE1bits_SSPIE = 1          ' Enable the SPI peripheral interrupt
    INTCON = $C0                ' Enable peripheral and global interrupts
    Return
'--------------------------------------------------------------------------
Main:
    GoSub SPI_SlaveInit
'
' Read the stream of SPI data and read or write accordingly
'
    While
        '
        ' Trap an unexpected activation of the SS pin
        '
        If SPI_SS = 1 Then                          ' Is the SS line high?
            PIR1bits_SSPIF = 0                      ' Yes. So clear the SPI flag
            WREG = SSPBUF                           ' Clear SSPBUF
            SPI_bByteCount = 0                      ' Reset the byte counter
            SPI_tDataReady = False                  ' Reset the ready flag
        EndIf
        '
        ' Read the data structure. i.e. Mnemonic, Address, Value
        '
        If SPI_tDataReady = True Then               ' Has a byte been read from the SPI interrupt?
            SPI_tDataReady = False                  ' Yes. So reset the ready flag
            If SPI_bByteCount = 1 Then              ' Have we read the first byte?
                SPI_bMnemonic = SPI_bValue          ' Yes. So this is the mnemonic

            ElseIf SPI_bByteCount = 2 Then          ' Have we read the second byte?
                SPI_bAddress = SPI_bValue           ' Yes. So this is the address
                If SPI_bMnemonic = cMnRead Then     ' Is the mnemonic a read command?
                    SSPBUF = bRAM[SPI_bAddress]     ' Yes. So setup for the return byte
                EndIf
            Else                                    ' Otherwise... It must be a third byte
                If SPI_bMnemonic = cMnWrite Then    ' Is the mnemonic a write command?
                    bRAM[SPI_bAddress] = SPI_bValue ' Yes. So place the byte read into the RAM array
                EndIf
                SPI_bByteCount = 0                  ' Reset the byte counter
            EndIf
        EndIf
    Wend
'--------------------------------------------------------------------------


